/*
libmatrix.c

matrix manipulation library
version 0.02
Peter A. Cejchan, GPL
Thu Mar 25 13:13:11 EST 2004 <cej@gli.cas.cz | pac7@post.cz | pac@plan9.cz>


*/


#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "libmatrix.h" 


// functions:
int 
readheader(int *rows, int *cols){
// reads #rows #columns

	char *buf, *av[LINELEN];
	int i, j, n;

	if((buf = Brdstr(&bin, '\n', 1)) != nil){
		// skip comment lines
		while((buf[0]=='#') || (buf[0]=='>')){

			print("comments\n");	
			if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;	// read another line
		}	

		n=tokenize(buf, av, 2);
		// skip whitespace lines
		while(n==0){
			print("whitespace\n");	
			if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;	// read another line
			n=tokenize(buf, av, 2);
		}	
		*rows=atoi(av[0]);
		*cols=atoi(av[1]);
//print("%s %s\n", av[0], av[1]);	
			return 1;
	}
}


int
readmat(Matrix *mat){
// reads in a whitespace and newline separated matrix

	char *buf, *av[LINELEN];
	int i, j, n;
	for(i=0; i<mat->rows; i++){
		if((buf = Brdstr(&bin, '\n', 1)) != nil){	


			// skip comment lines
			while((buf[0]=='#') || (buf[0]=='>')){

				print("comments\n");	
				if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;	// read another line
			}	

			n=tokenize(buf, av, mat->cols);
			// skip whitespace lines
			while(n==0){
				print("whitespace\n");	
				if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;	// read another line
				n=tokenize(buf, av, mat->cols);
			}	


			for(j=0; j<mat->cols; j++)
				mat->x[i][j]=atof(av[j]);


			free(buf);
		

		}
		else return 0;
		
	}
	return 1;
}


Matrix *
allocmat(int rows, int cols){
// allocates memory for a matrix

	int i,j;
	Matrix *M;

	M=(Matrix*)malloc(sizeof(Matrix));
	M->rows=rows; M->cols=cols;

	M->x=(double**)malloc(M->rows*sizeof(double*));
	for(j=0;j<rows;j++) M->x[j]=(double*)malloc(cols*sizeof(double));

	for(i=0;i<M->rows;i++)
		for(j=0;j<M->cols;j++) M->x[i][j]=0.0;
	return M;
}



void 
writemat(Matrix *mat){
	int i, j;
	print("%d %d ", mat->rows, mat->cols);
	for(i=0; i<mat->rows; i++){
		for(j=0; j<mat->cols; j++){
			print("%f ", mat->x[i][j]);
		}
		print("\n");
	}
	return;
}

void 
writeimat(iMatrix *mat){
	int i, j;
	print("%d %d ", mat->rows, mat->cols);
	for(i=0; i<mat->rows; i++){
		for(j=0; j<mat->cols; j++){
			print("%d ", mat->x[i][j]);
		}
		print("\n");
	}
	return;
}


int 
matmax(Matrix *mat){
//prints out row and col index of first occurrence of max value
	int i, j, maxi, maxj;
	double maxx;
	maxx = -MAXFLOAT;
	for(i=0; i<mat->rows; i++){
		for(j=0; j<mat->cols; j++){
//			print("%f ", mat->x[i][j]);
			if(mat->x[i][j] > maxx) {
				maxx = mat->x[i][j];
				maxi = i; maxj = j;
			}
		}
//		print("\n");
	}
//	print("%f\n", maxx);
	print("%d %d\n", maxi, maxj);
	return 1;
}

int 
mattranspose(Matrix *mat){
	Matrix tmp;
	int i, j;

	for(i=0; i<mat->rows; i++){
		for(j=0; j<mat->cols; j++){
			tmp.x[j][i] = mat->x[i][j];
		}
	}
	for(i=0; i<mat->rows; i++){
		for(j=0; j<mat->cols; j++){
			mat->x[i][j] = tmp.x[i][j];
		}
	}
	return 1;
}

int 
imattranspose(Matrix *mat){
	iMatrix tmp;
	int i, j;

	for(i=0; i<mat->rows; i++){
		for(j=0; j<mat->cols; j++){
			tmp.x[j][i] = mat->x[i][j];
		}
	}
	for(i=0; i<mat->rows; i++){
		for(j=0; j<mat->cols; j++){
			mat->x[i][j] = tmp.x[i][j];
		}
	}
	return 1;
}

void 
matmul(Matrix *a, Matrix *b, Matrix *out){
//untested!!! out= a*b

	int i, j, k;
	double sum;
	for(i=0;i<a->rows;i++) for(j=0;j<a->rows;j++){
		sum=0;
		for(k=0;k<a->cols;k++)
			sum+=a->x[i][k]*b->x[k][j];
		out->x[i][j]=sum;
	}
}

int 
readvec(Vector *vec){
	Biobuf bin;
	char *buf, *av[LINELEN];
	int i, j;

	Binit(&bin, 0, OREAD);
	i=0;
	if((buf = Brdstr(&bin, '\n', 1)) != nil){
		vec->length=   tokenize(buf, av, MAXLENGTH);
		for(j=0; j<vec->length; j++){
			vec->x[j]=atof(av[j]);
		}
		free(buf);
	}
	return 1;
}

int 
readivec(iVector *vec){
	Biobuf bin;
	char *buf, *av[LINELEN];
	int i, j;

	Binit(&bin, 0, OREAD);
	i=0;
	if((buf = Brdstr(&bin, '\n', 1)) != nil){
		vec->length=   tokenize(buf, av, MAXLENGTH);
		for(j=0; j<vec->length; j++){
			vec->x[j]=atoi(av[j]);
		}
		free(buf);
	}
	return 1;
}


int
neighborpair(iVector *vec, iMatrix *mat){
	int i, j;
	for(i=0; i<mat->rows; i++){
		for(j=0; j<mat->cols; j++){
			mat->x[i][j]=0;
		}
	}
	for(i=0; i<vec->length-1; i++){
		mat->x[vec->x[i]][vec->x[i+1]]=0;
		mat->x[vec->x[i+1]][vec->x[i]]=0;
	}

}


void 
ident(Matrix *mat){
// store identity matrix in mat
	int i, j;
	for(i=0; i<mat->rows; i++){
		for(j=0; j<mat->cols; j++){
			if(i==j){
				mat->x[j][i] = 1;
			}
			else{
				mat->x[j][i] = 0;
			}
		}
	}
}

